/*
* Far Named Folders 2.x
* Copyright (c) 2002-2004 by Victor Derevyanko, RAMMuS Group
* www: http://www.rammus.ru
* e-mail: wingspan@yandex.ru
*/
#include "StdAfx.h"
#include "executor_addons.h"

#include <cassert>
#include <vector>
#include <list>
#include <functional>
#include <shlwapi.h>
#include <stdlib.h>	//_environ

#include "executor.h"
#include "Kernel.h"
#include "stlcatalogs.h"
#include "executor_menus.h"
#include "lang.h"
#include "codec_for_values.h"
#include "header.h"
#include "executor_confirmations.h"
#include "strings_utils.h"
#include "Parser.h"


using namespace nf;

namespace
{
	struct cmp_strings 
		: public std::binary_function<tstring const&, tstring const&, bool>
	{
		bool operator()(tstring const& s1, tstring const &s2)
		{
			//return s1 < s2;
			return ! lstrcmpi(s1.c_str(), s2.c_str()) ? 1 : 0;
		}
	};

	struct file_not_exists
		: public std::unary_function<tstring const&, bool>
	{
		bool operator()(tstring const& s1)
		{
			//return s1 < s2;
			tstring::size_type npos = s1.find_first_of(_T("%"));
			TCHAR const* path = s1.c_str() + npos + 1;
			return ! ::PathFileExists(path);
		}
	};

	inline tstring extract_env_path(tstring var_and_path)
	{	//       
		size_t npos = var_and_path.find_first_of(_T('%'));
		if (npos != tstring::npos)
		{
			var_and_path.erase(var_and_path.begin(), var_and_path.begin() + npos + 1);
		}
		return var_and_path;
	}
}

bool nf::open_environment_variable(HANDLE hPlugin, tstring var_name)
{
	tstring result_path;
	if (select_path_for_environment_variable(hPlugin, var_name, result_path))
	//    
		return open_specified_path(hPlugin, result_path);			
	else
		return false;
};

bool nf::select_path_for_environment_variable(HANDLE hPlugin
											  , tstring const &var_name
											  , tstring &path)
{
	std::list<tstring> values;
	tstring mask = var_name;
	bool bAddAsterixToMask = true;
	if (var_name.size())
	{
		remove_leading_chars(mask, _T('%'));
		remove_trailing_chars(mask, _T(' '));
		//   %   * 
		//  -    %
		//    
		if (mask.size())
		{
			tstring::iterator p = mask.end()-1;
			if (*p == _T('%')) 
			{
				mask.erase(p);
				bAddAsterixToMask = false;	//    
			} 
		}
	}
	//    ,    var_name
	//         
	char **penv = _environ;
	while (*penv != NULL)
	{
		char *pe = *penv;
		while (*pe) if (*pe != _T('=')) ++pe; 
		else {pe++; break;}
		if (pe)
		{
			tstring env_name(*penv, pe - (*penv) - 1);
			if (parser::token_is_matched_to_pattern( env_name, mask, bAddAsterixToMask)) 
			{
				extract_paths(env_name, pe, values, _T(';'));
			}
		}
		++penv;
	}
	if (! values.size()) return false;

	//       ,   
	// FileExists
	values.remove_if(file_not_exists());
	values.sort(cmp_strings());
	values.unique(cmp_strings());

	//     
	tstring result_path;
	if (values.size() == 1) {
		result_path = *values.begin();
	} else {
		kr::tenvvars_list vv;
		std::copy(values.begin(), values.end(), std::insert_iterator<kr::tenvvars_list >(vv, vv.begin()));
		if (! select_required_envvar_path_through_menu(vv, result_path)) return false;
	}
	//    
	path = extract_env_path(result_path);
};


//      
//   ,    ...
bool nf::select_required_shortcut_by_path(HANDLE hPlugin
										  , tstring const &current_path
										  , kr::tparsed_command const &cmd
										  , kr::tshortcut_info& sh)
{
	while (true)
	{
		kr::tshortcuts_list list;
		kr::tvalues_list values;

		switch(cmd.kind) {
		case kr::QK_DELETE_SHORTCUT_IMPLICIT:
			g_pK->select_variants_shortcuts_by_path(_T("")
				, current_path.c_str()
				, true
				, list);
			break;
		case kr::QK_OPEN_BY_PATH:
			g_pK->select_variants_shortcuts_by_path(cmd.catalog.c_str()
				, cmd.shortcut.c_str()
				, false
				, list);
			break;
		default:
			return false;
		} 
		int nCodeRet = nf::select_required_shortcut_through_menu(list, values, sh);
		if (nCodeRet == -1)
		{
			//  ..
			delete_shortcut(hPlugin, sh, false);
			continue;
		} else return nCodeRet != 0;
	}; //while

}


bool nf::extract_paths(tstring const& EnvVarName
					   , tstring var
					   , std::list<tstring> &ExtractedPaths
					   , TCHAR delimeter)
{
	size_t off = 0;
	size_t off2 = 0;
	if (! var.size()) return false;
	if (*(var.end()-1) != _T(';')) var += _T(";");

	while ((off2 = var.find(delimeter, off)) != tstring::npos)
	{
		tstring v;
		v.append(var.begin() + off, var.begin() + off2);
	//   :   % 
		v = EnvVarName + _T("%") + v;
		ExtractedPaths.push_back(v);
		off = off2+1;
	}
	return ExtractedPaths.size();
}