/*
* Far Named Folders 2.x
* Copyright (c) 2002-2004 by Victor Derevyanko, RAMMuS Group
* www: http://www.rammus.ru
* e-mail: wingspan@yandex.ru
*/
#include "StdAfx.h"
#include "executor_menus.h"
#include <vector>
#include <cassert>
#include "stlcatalogs.h"
#include "shlwapi.h"

//  Loki  
#include <functor.h>
#include <Typelist.h>

using namespace nf;

#pragma comment(lib, "shlwapi.lib")

template<class Value, class List>
class CMenuT 
{
public:
	typedef Loki::Functor<int, TYPELIST_2(int, Value const&) > tactor_type;
		//,   ..
		//   ,    
		// .
	typedef Loki::Functor<void, TYPELIST_2(std::vector<FarMenuItem>&, List const&) > tloader_type;
		//,    

	CMenuT(tloader_type Loader
		, tactor_type *pActor
		, int *pBreakKeys
		, TCHAR const* KeysImMenu 
		, TCHAR const* HelpTopic
		) 
		: m_Loader(Loader)
		, m_pActor(pActor)
		, m_pBreakKeys(pBreakKeys)
		, m_KeysInMenu(KeysImMenu)
		, m_HelpTopic(HelpTopic)
	{
	}

	int Display(List const &list, Value &c)
	{
		if (! list.size()) return 0;	//  
		if (1 == list.size()) 
		{
			c = *list.begin();
			return 1;		//  -    ..
		}

		while (true)
		{
			std::vector<FarMenuItem> MenuItems(list.size());
			m_Loader(MenuItems, list);

			int BreakCode = 0;
			int nSelectedItem = g_PluginInfo.Menu(
				g_PluginInfo.ModuleNumber
				, -1
				, -1
				, 0	// -  
				, FMENU_AUTOHIGHLIGHT | FMENU_WRAPMODE
				, ""
				, m_KeysInMenu.c_str()
				, m_HelpTopic.c_str()
				, m_pBreakKeys 
				, &BreakCode
				, &MenuItems[0]
				, static_cast<int>(MenuItems.size())
			);

			if (-1 == nSelectedItem) return 0;	//user has canceled menu
			List::const_iterator p = list.begin();
			for (int i = 0; i < nSelectedItem; ++i) ++p;
			c = *p;

			if (BreakCode == -1) break; //   

			if (m_pActor) 
			{
				int nret = (*m_pActor)(BreakCode, c);
				if (nret < 0) 
					return nret;	// "" (,   )
			}
		};

		return 1;
	}
private:
	tloader_type m_Loader;
	tactor_type *m_pActor;
	int *m_pBreakKeys;
	tstring m_HelpTopic;
	tstring m_KeysInMenu;
};

namespace {
	inline void safe_copy(char *t, tstring const &Src, UINT MaxSize)
	{
		if (static_cast<UINT>(Src.size()) > MaxSize)
		{
			PathCompactPathEx(t, Src.c_str(), MaxSize, 0);
		} else 
			lstrcpy(t, Src.c_str());
	}

	void load_shortcuts_values(kr::tshortcuts_list const &list, kr::tvalues_list &values)
	{	//      list
		values.clear();

		kr::tshortcuts_list::const_iterator p1 = list.begin();
		while (p1 != list.end())
		{
			kr::tshortcut_info const &sh = *p1;
			tstring value;
			sc::CCatalog c(sh.catalog);
			if (c.get_shortcut_info(sh.shortcut, sh.bIsTemporary, value))
			{
				values.push_back(value);
			} else {
				assert(false);
				values.push_back(_T(""));
			}
			++p1;
		}
	}

	template <class T, class F> 
	UINT GetMaxWidth(F f, T list)
	{
		UINT width = 0;
		typename T::const_iterator p = list.begin();
		while (p != list.end())
		{
			UINT w = static_cast<UINT>(f(list, p).size());
			if (w > width) width = w;
			++p;
		}
		return width;
	}


	class CShName
	{
		kr::tvalues_list const &m_values;
	public:
		typedef kr::tshortcuts_list::const_iterator Iter;
		CShName(kr::tvalues_list const &values) : m_values(values) {}
		inline tstring const& GetShortcutName(kr::tshortcuts_list const &, Iter p) const 
		{
			return p->shortcut;
		}
		inline tstring const& GetCatalogName(kr::tshortcuts_list const &, Iter p) const 
		{
			return p->catalog;
		}
		inline tstring GetShortcutAndCatalog(kr::tshortcuts_list const &, Iter p) const 
		{
			return (p->catalog.size()) 
				? p->catalog + SLASH_CATS + p->shortcut
				: p->shortcut;
		}
		inline tstring const& GetValue(kr::tshortcuts_list const & list, Iter p) const 
		{
			return m_values[p - list.begin()];
		}
	};

	struct CEnvName
	{
		typedef kr::tenvvars_list::const_iterator Iter;
		inline tstring GetEnvVarName(kr::tenvvars_list const &, Iter p) const 
		{
			tstring s = *p;
			size_t npos = s.find_first_of(_T('%'));
			if (npos == tstring::npos) return s;
			s.erase(s.begin() + npos, s.end());
			return s;
		}
		inline tstring GetPath(kr::tenvvars_list const &, Iter p) const 
		{
			tstring s = *p;
			size_t npos = s.find_first_of(_T('%'));
			if (npos == tstring::npos) return s;
			s.erase(s.begin(), s.begin() + npos + 1);
			return s;
		}
	}; 

//////////////////////////////////////////////////////////////////////////
	template <class List>
	class CCopierT
	{
		std::vector<FarMenuItem>& m_MenuItems;
		List const &m_list;
	public:
		typedef typename List::const_iterator Iterator;
		typedef Loki::Functor<tstring, TYPELIST_2(List, Iterator)> titem;
		enum {MAX_FIRST_COLUMN_WIDTH = 50, MAX_WIDTH = 128};

		CCopierT(std::vector<FarMenuItem>& MenuItems, List const& list)
			: m_MenuItems(MenuItems), m_list(list) {}
		inline void SafeCopy(titem f)
		{
			int i = 0;
			for (Iterator p = m_list.begin(); p != m_list.end(); ++p, ++i)
			{
				m_MenuItems[i].Checked = 0;
				m_MenuItems[i].Selected = 0;
				m_MenuItems[i].Separator = 0;
				safe_copy(&m_MenuItems[i].Text[0], f(m_list, p), MAX_WIDTH);
			}
		}
		inline void SafeCopy(titem f1, titem f2, UINT FirstColumnMaxWidth)
		{
			UINT width = (FirstColumnMaxWidth > MAX_FIRST_COLUMN_WIDTH)
				? MAX_FIRST_COLUMN_WIDTH
				: FirstColumnMaxWidth;
			int i = 0;
			for (Iterator p = m_list.begin(); p != m_list.end(); ++p, ++i)
			{
				m_MenuItems[i].Checked = 0;
				m_MenuItems[i].Selected = 0;
				m_MenuItems[i].Separator = 0;
				tstring s1 = f1(m_list, p);
				safe_copy(&m_MenuItems[i].Text[0], s1, MAX_WIDTH);
				memset(&m_MenuItems[i].Text[s1.size()], _T(' '), width-s1.size()+1);
				safe_copy(&m_MenuItems[i].Text[width+1], f2(m_list, p), MAX_WIDTH - width - 1);
			}
		}
	};

	struct CEvnManager
	{
		enum {NUMBER_MENU_MODES = 2};	 //-   
		void load(std::vector<FarMenuItem>& MenuItems, kr::tenvvars_list const &list)
		{
			assert(MenuItems.size() == list.size());
			enum {COL_VAR = 0x1, COL_PATH = 0x2};
			const int MODES[NUMBER_MENU_MODES] = {COL_VAR & COL_PATH, COL_PATH};
			int CurrentMenuMode = g_pK->m_S.GetValue(nf::ST_SELECT_SH_MENU_MODE_EV);	
			if ((CurrentMenuMode > 2) || (CurrentMenuMode < 1)) CurrentMenuMode = 2;
			int CurrentMask = MODES[CurrentMenuMode-1];		

			CEnvName namer;
			typedef CCopierT<kr::tenvvars_list > CCopier;
			CCopier::titem fvar(&namer, &CEnvName::GetEnvVarName);
			CCopier::titem fpath(&namer, &CEnvName::GetPath);

			CCopier mc(MenuItems, list);
			switch (CurrentMenuMode)
			{
			case 2: mc.SafeCopy(fpath); break;
			default:  mc.SafeCopy(fvar, fpath, GetMaxWidth(fvar, list));
			};
		} //load

		int action(int BreakCode, tstring const& SelectedItem)
		{
			//   
			//    
			g_pK->m_S.SetValue(nf::ST_SELECT_SH_MENU_MODE_EV, BreakCode+1);	
			g_pK->m_S.save();
			return 0;
		}
	};

	class shortcuts_manager
	{
		kr::tvalues_list &m_values;
	public:	
		shortcuts_manager(kr::tvalues_list &values) : m_values(values) {}

		enum {NUMBER_MENU_MODES = 5};	 //-   
		void load(std::vector<FarMenuItem>& MenuItems, kr::tshortcuts_list const &list)
		{
			assert(MenuItems.size() == list.size());
			enum {	//   
				COL_SH		= 0x1
				, COL_VAL	= 0x2
				, COL_CAT	= 0x4
			};
			//    - 1,2,3,4,5
			// action   .
			const int MODES[NUMBER_MENU_MODES] =//  
			{									//    
				COL_SH							// 
				, COL_VAL						// 
				, COL_SH | COL_VAL				//  
				, COL_CAT | COL_SH				//  
				, COL_CAT | COL_SH | COL_VAL	//,   
			};
			int CurrentMenuMode = g_pK->m_S.GetValue(nf::ST_SELECT_SH_MENU_MODE);	//   
			if ((CurrentMenuMode > 5) || (CurrentMenuMode < 1)) CurrentMenuMode = 1;
			
			int CurrentMask = MODES[CurrentMenuMode-1];		
			if (CurrentMask & COL_VAL) load_shortcuts_values(list, m_values);	//   
			
			CShName namer(m_values);
			typedef CCopierT<kr::tshortcuts_list> CCopier;
			CCopier::titem fsh(&namer, &CShName::GetShortcutName);
			CCopier::titem fshcat(&namer, &CShName::GetShortcutAndCatalog);
			CCopier::titem fval(&namer, &CShName::GetValue);	
			
			CCopier mc(MenuItems, list);
			switch (CurrentMenuMode)
			{
			case 2: mc.SafeCopy(fval); break;
			case 3:  mc.SafeCopy(fsh, fval, GetMaxWidth(fsh, list)); break;
			case 4:  mc.SafeCopy(fshcat); break;
			case 5:  mc.SafeCopy(fshcat, fval, GetMaxWidth(fshcat, list)); break;
			default:  mc.SafeCopy(fsh);
			};
		} //load

		int action(int BreakCode, kr::tshortcut_info const& SelectedShortcut)
		{
			if (BreakCode == 0)
			{	//Del -   
				return -1;
			} else {
				//   
				//    
				g_pK->m_S.SetValue(nf::ST_SELECT_SH_MENU_MODE, BreakCode);	
				g_pK->m_S.save();
			}
			return 0;
		}
	}; //class


	template<class List>
	void direct_loader(std::vector<FarMenuItem>& MenuItems, List const& list)
	{
		assert(MenuItems.size() == list.size());
		typename List::const_iterator p = list.begin();
		int i = 0;
		while (p != list.end())
		{
			safe_copy(MenuItems[i].Text, *p, 127);
			MenuItems[i].Selected = 0;
			MenuItems[i].Checked = 0;
			MenuItems[i].Separator = 0;
			++i;
			++p;
		}
	}

	template void direct_loader<>(std::vector<FarMenuItem>&, kr::tdirs_list const& list);
	template void direct_loader<>(std::vector<FarMenuItem>&, kr::tcatalogs_list const& list); 
	template void direct_loader<>(std::vector<FarMenuItem>&, kr::tenvvars_list const& list); 

}; //namespace

//////////////////////////////////////////////////////////////////////////

bool nf::select_required_envvar_path_through_menu(kr::tenvvars_list const &list
												  , tstring &result_path)
{
	const NUM_BREAK_KEYS = 3;
	int BreakKeys[NUM_BREAK_KEYS] =	
		{VK_F2, VK_F3, 0};	//   :     

	typedef CMenuT<tstring, kr::tenvvars_list > CMenu;
	::CEvnManager m;
//	CMenu::tloader_type floader(&direct_loader<tstring>);
	CMenu::tloader_type floader(&m, ::CEvnManager::load);
	CMenu::tactor_type factor(&m, ::CEvnManager::action);
	CMenu menu(floader, &factor, &BreakKeys[0], _T("F2-F3"), _T(""));
	return menu.Display(list, result_path);
}

int nf::select_required_catalog_through_menu(kr::tcatalogs_list const &list	//  
											 , kr::tcatalog_info &result_catalog	// 
											 )
{
	typedef CMenuT<kr::tcatalog_info, kr::tcatalogs_list > CMenu;
	CMenu::tloader_type floader(&direct_loader<kr::tcatalogs_list>);
	CMenu menu(floader, 0, 0, _T(""), _T(""));
	return menu.Display(list, result_catalog);
}


bool nf::select_required_path_through_menu (kr::tdirs_list const &list
											, tstring &result_path)
{
	typedef CMenuT<tstring, kr::tdirs_list > CMenu;
	CMenu::tloader_type floader(&direct_loader<kr::tdirs_list>);
	CMenu menu(floader, 0, 0, _T(""), _T(""));
	return menu.Display(list, result_path);
}

int nf::select_required_shortcut_through_menu(kr::tshortcuts_list const &list
											  , kr::tvalues_list &values
											  , kr::tshortcut_info& sh)
{
	typedef CMenuT<kr::tshortcut_info, kr::tshortcuts_list > CMenu;
	::shortcuts_manager m(values);
	CMenu::tloader_type floader(&m, &::shortcuts_manager::load);
	CMenu::tactor_type factor(&m, &::shortcuts_manager::action);

	const NUM_BREAK_KEYS = 8;
	int BreakKeys[NUM_BREAK_KEYS] =	//     	
	{VK_DELETE, VK_F2, VK_F3, VK_F4, VK_F5, VK_F6, VK_SPACE, 0};	

	CMenu menu(floader, &factor, BreakKeys, _T("Del, F2-F6"), _T("MenuSelect"));
	int nret =  menu.Display(list, sh);
	return nret;
} 